#include <SoftwareSerial.h>
#include <Servo.h>

#define BT_TX_PIN 2  //TX pin of bluetooth module
#define BT_RX_PIN 3  //RX pin of bluetooth module
#define LEFT_WHEEL_PIN 9
#define RIGHT_WHEEL_PIN 5
#define LEFT_ARM_PIN 6
#define RIGHT_ARM_PIN 3
#define CLK_PIN 11
#define CS_PIN 10
#define DIN_PIN 12
#define BUZZER_PIN 8

SoftwareSerial _bt_device(BT_TX_PIN,BT_RX_PIN);
Servo leftWheel;
Servo rightWheel;
Servo leftArm;
Servo rightArm;

int _bt_pos=0;
unsigned char _bt_cmd=0;
int _bt_length=0;
unsigned char _bt_data[255];

#define CMD_SERVO 0x10
#define CMD_SERVO_CONT 0x11
#define CMD_BUZZER_MELODY 0x21
#define CMD_LED_MATRIX_PREDEF_EXPR 0x51


const uint16_t GAME_THRONES[] = {1568,562,1047,562,1245,93,1397,93,1568,375,1047,375,1245,93,1397,93,1175,1875,1397,562,932,562,1245,93,1175,93,1397,375,932,562,1245,93,1175,93,1047,750};
const uint16_t IMPERIAL_MARCH[] = {440,375,440,375,440,375,349,281,523,93,440,375,349,281,523,93,440,750,659,375,659,375,659,375,698,281,523,93,415,375,349,281,523,93,440,750,880,375,440,281,440,93,880,375,831,281,784,93,740,93,698,93,740,187,0,187,446,281,622,375,587,281,554,93,523,93,494,93,523,187,0,187,349,187,415,375,349,281,392,93,523,375,440,281,523,93,659,750,880,375,440,281,440,93,880,375,831,281,784,93,740,93,698,93,740,187,0,187,446,281,622,375,587,281,554,93,523,93,494,93,523,187,0,187,330,187,415,375,330,281,523,93,440,375,349,281,523,93,440,750};
const uint16_t SEVEN_NATIONS_ARMY[] = {330,759,330,252,392,378,330,125,0,252,294,252,262,1012,245,1012,330,759,330,252,392,378,330,125,0,252,294,252,262,506,294,252,262,252,245,1012,330,759,330,252,392,378,330,125,0,252,294,252,262,1012,245,1012};

void LEDMatrix_init(int cs, int din, int clk);
void writeRow(int cs, int din, int clk, int row, int data);
void maxAll (int cs, int din, int clk, int reg, int col);
void putByte (int din, int clk, int data);
void expression(int cs, int din, int clk, int col1, int col2, int col3, int col4, int col5, int col6, int col7, int col8);
void playMelody(int pin,const uint16_t* melody, int length);
void happy ();
void sad ();
void love ();
void angry ();
void stunned ();

void setup()
{
  _bt_device.begin(9600);
  _bt_device.flush();
  pinMode(BUZZER_PIN,OUTPUT);
  pinMode(CLK_PIN,OUTPUT);
  pinMode(CS_PIN,OUTPUT);
  pinMode(DIN_PIN,OUTPUT);
  //Set the LED Matrix component
  maxAll(CS_PIN,DIN_PIN,CLK_PIN,11,7);
  maxAll(CS_PIN,DIN_PIN,CLK_PIN,9,0);
  maxAll(CS_PIN,DIN_PIN,CLK_PIN,12,1);
  maxAll(CS_PIN,DIN_PIN,CLK_PIN,15,0);
  for (int i = 1; i <= 8; i++)
    maxAll(CS_PIN,DIN_PIN,CLK_PIN,i,0);
  maxAll(CS_PIN,DIN_PIN,CLK_PIN,10,15);
  expression(CS_PIN,DIN_PIN,CLK_PIN,0,0,0,0,0,0,0,0);
  
  leftWheel.attach(LEFT_WHEEL_PIN);
  delay(300);
  rightWheel.attach(RIGHT_WHEEL_PIN);
  delay(300);
  leftArm.attach(LEFT_ARM_PIN);
  delay(300);
  rightArm.attach(RIGHT_ARM_PIN);
  delay(300);
}
void loop()
{
  if (_bt_device.available()>0)
  {
    unsigned char c;
    _bt_device.readBytes(&c,1);
    if ((c=='@')&&(_bt_pos==0))
      _bt_pos++;
    else if (_bt_pos==1){
      _bt_pos++;
      _bt_cmd=c;
    }
    else if (_bt_pos==2){
      _bt_pos++;
      _bt_length=c;
    }
    else if ((_bt_pos>=3)&&(_bt_pos<=(_bt_length+2))){
      _bt_data[_bt_pos-3]=c;
      _bt_pos++;
    }
    else if ((_bt_pos==(_bt_length+3))&&(c=='*')){
      if (_bt_cmd==CMD_SERVO_CONT){
        int pin = _bt_data[0];
        byte value = _bt_data[1];
        if (pin==LEFT_WHEEL_PIN)
          leftWheel.write(((value*90)/100+90));
        else if (pin==RIGHT_WHEEL_PIN)
          rightWheel.write(((value*90)/100+90));
      }
      if (_bt_cmd==CMD_SERVO){
        int pin = _bt_data[0];
        byte value = _bt_data[1];
        if (pin==LEFT_ARM_PIN)
          leftArm.write(value);
        else if (pin==RIGHT_ARM_PIN)
          rightArm.write(value);
      }
      if (_bt_cmd==CMD_LED_MATRIX_PREDEF_EXPR){
        int pin_clk = _bt_data[0];
        int pin_cs = _bt_data[1];
        int pin_din = _bt_data[2];
        byte number = _bt_data[3];
        if ((pin_clk==CLK_PIN)&&(pin_cs==CS_PIN)&&(pin_din==DIN_PIN)){
          switch (number)
          {
            case 1:
              happy();
              break;
            case 2:
              sad();
              break;
            case 3:
              love();
              break;
            case 4:
              angry();
              break;
            case 5:
              stunned();
              break;
          }
        }
      }
      if (_bt_cmd==CMD_BUZZER_MELODY){
        int pin = _bt_data[0];
        byte melody = _bt_data[1];
        if (pin==BUZZER_PIN){
          switch (melody)
          {
            case 1:
              playMelody(BUZZER_PIN,GAME_THRONES,sizeof(GAME_THRONES)/(2*sizeof(uint16_t)));
              break;
            case 2:
              playMelody(BUZZER_PIN,IMPERIAL_MARCH,sizeof(IMPERIAL_MARCH)/(2*sizeof(uint16_t)));
              break;
            case 3:
              playMelody(BUZZER_PIN,SEVEN_NATIONS_ARMY,sizeof(SEVEN_NATIONS_ARMY)/(2*sizeof(uint16_t)));
              break;
          }
        }
      }
      _bt_pos=0;
      _bt_length=0;
    }
    else{
      _bt_pos=0;
      _bt_length=0;
    }
  }
}

void LEDMatrix_init(int cs, int din, int clk) {
  maxAll(cs,din,clk,11,7);
  maxAll(cs,din,clk,9,0);
  maxAll(cs,din,clk,12,1);
  maxAll(cs,din,clk,15,0);
  for (int i = 1; i <= 8; i++)
    maxAll(cs,din,clk,i, 0);
  maxAll(cs,din,clk,10,15);
}

void writeRow(int cs, int din, int clk, int row, int data) {
  digitalWrite(cs,LOW);
  putByte(din,clk,row);
  putByte(din,clk,data);
  digitalWrite(cs,LOW);
  digitalWrite(cs,HIGH);
}

void maxAll (int cs, int din, int clk, int reg, int col) {
  digitalWrite(cs,LOW);
  putByte(din,clk,reg);
  putByte(din,clk,col);
  digitalWrite(cs,LOW);
  digitalWrite(cs,HIGH);
}
void putByte (int din, int clk, int data) {
  byte i = 8;
  byte mask;
  while(i > 0) {mask = 0x01 << (i - 1);
    digitalWrite(clk,LOW);
    if (data & mask)
  {
      digitalWrite(din,HIGH);
    }else{
      digitalWrite(din,LOW);
    }
    digitalWrite(clk,HIGH);
    --i;
  }
}

void expression(int cs, int din, int clk, int col1, int col2, int col3, int col4, int col5, int col6, int col7, int col8) {
  writeRow(cs,din,clk,1, col1);
  writeRow(cs,din,clk,2, col2);
  writeRow(cs,din,clk,3, col3);
  writeRow(cs,din,clk,4, col4);
  writeRow(cs,din,clk,5, col5);
  writeRow(cs,din,clk,6, col6);
  writeRow(cs,din,clk,7, col7);
  writeRow(cs,din,clk,8, col8);
}

void playMelody(int pin,const uint16_t* melody, int length)
{
  unsigned int note;
  unsigned long duration;
  uint16_t* melody_ptr=(uint16_t*)melody;
  for (int i=0;i<length;i++)
  {
    note=*melody_ptr++;
    duration=*melody_ptr++;
    tone(pin,note,duration);
    delay(duration);
    noTone(pin);
  }
}

void happy () {
  expression(10,12,11,12,24,48,48,48,48,24,12);
}

void sad () {
  expression(10,12,11,48,24,12,12,12,12,24,48);
}

void love () {
  expression(10,12,11,12,30,62,124,124,62,30,12);
}

void angry () {
  expression(10,12,11,2,68,36,16,16,36,68,2);
}

void stunned () {
  expression(10,12,11,0,0,0,24,24,0,0,0);
}
